# Impute missing data, itself from a random forest
invisible(capture.output(data_imp_train <- missForest(vir_train[, all.vars(formula_used)])))
vir_train <- data_imp_train$ximp
invisible(capture.output(data_imp_test <- missForest(vir_test[, all.vars(formula_used)])))
vir_test <- data_imp_test$ximp

# Build random forest model from 5000 individual bootstrapped trees
random_forest <- randomForest(formula_used, 
                              dat= vir_train,
                              importance = TRUE, 
                              na.action = na.exclude, 
                              keep.forest = TRUE, 
                              ntree = 5000, 
                              proximity = TRUE,
                              nodesize = 5)

# Extract variable importances and calculate relative mean decrease in Gini coefficient associated with each predictor
varimp <- as.data.frame(random_forest$importance)
varimp$relGini <- varimp$MeanDecreaseGini/max(varimp$MeanDecreaseGini)
# Append variable names to data frame
varimp$name <- rownames(varimp)
rownames(varimp) <- NULL

# Extract partial dependences as marginal relative change in log-odds of severe disease, and as marginal probability of severe disease
list_PD <- list_prob <<- vector("list", length(all.vars(formula_used))-1) 
names(list_PD) <- names(list_prob) <- all.vars(formula_used)[2:length(all.vars(formula_used))]

for (i in 2:length(all.vars(formula_used))){
  list_PD[[i-1]] <- partial(random_forest, pred.var=all.vars(formula_used)[i], which.class="severe")
}

for (i in 2:length(all.vars(formula_used))){
  list_prob[[i-1]] <- partial(random_forest, pred.var=all.vars(formula_used)[i], which.class="severe", prob=TRUE)
}

# Arrange partial dependences into data frame
df_PD_prob <- data.frame(Trait = unlist(data.frame(t(sapply(list_prob,c)))$Family),
                         PD = round(unlist(data.frame(t(sapply(list_PD,c)))$yhat),3),
                         Prob = round(unlist(data.frame(t(sapply(list_prob,c)))$yhat),3))
df_PD_prob <- data.frame(Variable = gsub('[[:digit:]]+', '', rownames(df_PD_prob)), df_PD_prob)
rownames(df_PD_prob) <- NULL

# Prediction diagnostics and accuracy statistics for test set
predict_forest_class_test <- predict(random_forest, newdata=vir_test, type="response")
predict_forest_prob_test <- predict(random_forest, newdata=vir_test, type="prob")
matrix_forest_test <- confusionMatrix(predict_forest_class_test, vir_test$Virulence, positive="severe")

# Calculate ROC curve for random forest
roc_pred_forest <- prediction(predict_forest_prob_test[,2], as.numeric(vir_test$Virulence)-1)
# Calculate AUROC for random forest
AUC_class_forest <- performance(roc_pred_forest, measure="auc")@y.values
